function vis_goplay()
	-- First, find out where we'd like to spawn.
	local start = nil
	if count.startpoint ~= nil and math.floor(entitydata[count.startpoint].x / 40) == roomx and math.floor(entitydata[count.startpoint].y / 30) == roomy then
		start = {
			(entitydata[count.startpoint].x-roomx*40)*8,
			(entitydata[count.startpoint].y-roomy*30)*8,
			false,
			1-entitydata[count.startpoint].p1
		}
	else
		for k,v in pairs(entitydata) do
			if (v.t == 10) and (v.x >= roomx*40) and (v.x <= (roomx*40)+39) and (v.y >= roomy*30) and (v.y <= (roomy*30)+29) then
				if v.p1 == 0 then -- upside down
					start = {
						(v.x-roomx*40)*8,
						(v.y-roomy*30)*8,
						true,
						1
					}
				else
					start = {
						(v.x-roomx*40)*8,
						(v.y-roomy*30)*8-8,
						false,
						1
					}
				end
				break
			end
		end
	end

	if start == nil then
		temporaryroomname = "ERROR: No checkpoint to spawn at"
		temporaryroomnametimer = 90
	else
		-- Maybe we haven't seen this page yet.
		if not vis_return_to_6 and not s.vis_firstseen then
			to_astate("davvis", 2)
			return
		end

		vis_setVsync(false)
		love.keyboard.setKeyRepeat(false)

		to_astate("davvis", 0)

		-- Initialize a few variables
		vis_flashtimer = 0
		vis_shaketimer = 0
		vis_cutscenebars = 0
		vis_cutscenebars_dir = false

		-- How many collectables do we have?
		vis_n_trinkets, vis_trinkets_total = 0, count.trinkets
		vis_n_crewmates, vis_crewmates_total = 0, count.crewmates
		-- Keep track of which ones we have collected
		vis_trinkets_coll = {}
		vis_crewmates_coll = {}

		vvvvvv_textboxes = {}

		vis_player = vPlayer:new{x=start[1]+2, y=start[2], flipped=start[3], dir=start[4], is_main_player=true}
		--vis_player:setcheckpoint()

		-- Load the room! (That means, all the entities!)
		vis_init_room()

		vis_scripting = vScripting:new()
		vis_scripting.flags = {}

		-- Do we have music?
		if tonumber(metadata.levmusic) ~= 0 then
			vis_playmusic(metadata.levmusic, false)
		end
	end
end

function vis_animate(frames, frametime) -- 2 seconds must be divisible by the total animation time
	return frames[math.floor((vis_timer%(#frames*frametime))/frametime+1)]
end

function vis_init_room()
	vis_load_entities()
end

function vis_load_entities()
	vis_solid_entities = {} -- requires x, y, w, h, destroyed
	vis_unsolid_entities = {}
	vis_warp_lines_exist = false

	for k,v in pairs(entitydata) do
		if (v.x >= roomx*40) and (v.x <= (roomx*40)+39) and (v.y >= roomy*30) and (v.y <= (roomy*30)+29) then
			local finx, finy = (v.x-roomx*40)*8, (v.y-roomy*30)*8
			if v.t == 1 then
				-- Enemy
				table.insert(vis_unsolid_entities, vEnemy:new{x=finx, y=finy, p1=v.p1})
			elseif v.t == 2 then
				-- Platform (conveyor or moving)
				if v.p1 <= 4 then
					-- Moving
					table.insert(vis_solid_entities, vMovingPlat:new{x=finx, y=finy, p1=v.p1})
				else
					-- Conveyor
					table.insert(vis_solid_entities, vConveyor:new{x=finx, y=finy, p1=v.p1})
				end
			elseif v.t == 3 then
				-- Disappear
				table.insert(vis_solid_entities, vDisappearingPlat:new{x=finx, y=finy})
			elseif v.t == 9 then
				-- Trinket, but only if we haven't collected it
				if vis_is_uncollected(vis_trinkets_coll, finx, finy) then
					table.insert(vis_unsolid_entities, vTrinket:new{x=finx, y=finy})
				end
			elseif v.t == 10 then
				-- Checkpoint
				table.insert(vis_unsolid_entities, vCheckpoint:new{x=finx, y=finy, p1=v.p1, active=(
							    vis_player.checkpointrx == roomx
							and vis_player.checkpointry == roomy
							and vis_player.checkpointx == finx+2
							and vis_player.checkpointy == finy-(v.p1 == 1 and 8 or 0)
							and vis_player.checkpointz == (v.p1 == 0)
						)
					}
				)
			elseif v.t == 11 then
				-- Gravity line
				table.insert(vis_unsolid_entities, vGravityline:new{x=finx, y=finy, p1=v.p1, p2=v.p2, p3=v.p3})
			elseif v.t == 13 then
				-- Warp token
				table.insert(vis_unsolid_entities, vWarptoken:new{x=finx, y=finy, dx=v.p1, dy=v.p2})
			elseif v.t == 15 then
				-- Rescuable crewmate
				if vis_is_uncollected(vis_crewmates_coll, finx, finy) then
					local col = ({"cyan", "purple", "yellow", "red", "green", "blue"})[v.p1+1]
					table.insert(vis_unsolid_entities, vPlayer:new{
							x=finx+2, y=finy,
							mood=1,
							ai="faceplayer",
							is_main_player=false,
							rescuable=true,
							color=col,
							defaultcolor=col,
							flippable=true
						}
					)
				end
			elseif v.t == 16 then
				-- Start point
				-- Not handled here!
			elseif v.t == 17 then
				-- Roomtext
				table.insert(vis_unsolid_entities, vRoomtext:new{x=finx, y=finy, data=v.data})
			elseif v.t == 18 then
				-- Terminal
				table.insert(vis_unsolid_entities, vTerminal:new{x=finx, y=finy, script=v.data})
			elseif v.t == 19 then
				-- Script box
				table.insert(vis_unsolid_entities, vScriptbox:new{x=finx, y=finy, p1=v.p1, p2=v.p2, script=v.data})
			elseif v.t == 50 then
				-- Warp line
				table.insert(vis_unsolid_entities, vWarpline:new{x=finx, y=finy, p1=v.p1, p2=v.p2, p3=v.p3})
				vis_warp_lines_exist = true
			end
		end
	end
end

function vis_crash(message)
	-- This is automatically cleared when no longer necessary
	love.graphics.setScissor()
	mapscreenshot = love.graphics.newImage(love.graphics.newScreenshot())

	vis_crash_x, vis_crash_y = 238, 108
	vis_crash_movingwindow = false
	vis_crash_message = message
	--vis_crash_image = vis_crashmsg_2
	vis_crash_image = "2"
	vis_crash_holdingbtn = 0
	vis_crash_time = love.timer.getTime()

	love.audio.stop()

	to_astate("davvis", 1)

	cons("The error is: " .. message)
end

function vis_setVsync(val)
	--local za,zb,zc = love.window.getMode()
	--zc.vsync = val
	--love.window.setMode(za,zb,zc)
end

function vis_drawbackground(bg, metadata)
	local tils = metadata.tileset
	local tilc = metadata.tilecol

	if bg == "stars" then
		
	elseif bg == "falling" then
	
	elseif bg == "lab" then
		if tilc == 6 then
			-- Rainbow BG
			love.graphics.setColor(vis_labbgcolors[(math.floor(love.timer.getTime())%6)+1][2])
		else
			love.graphics.setColor(vis_labbgcolors[tilc+1][2])
		end
		love.graphics.rectangle("fill", screenoffset, 0, 640, 480)

		-- Having the blocks do strange things is only funnier, amirite?
		for block = 0, 31 do
			local direction = block % 4 -- 4
			local fixedaxis = block/31*307200
			local x, y
			local w, h
			if direction <= 1 then
				--  -> / <-
				w, h = 64, 24
				x, y = (love.timer.getTime()*(block*5000 % 640) % 804)-64, (fixedaxis%480)-12
				if direction == 1 then
					x = 704-x
				end
			else
				-- v / ^
				w, h = 24, 64
				x, y = (fixedaxis%640)-32, (love.timer.getTime()*(block*3750 % 480) % 628)-24
				if direction == 3 then
					y = 528-y
				end
			end

			if tilc == 6 then
				-- Rainbow BG
				love.graphics.setColor(vis_labbgcolors[(math.floor(love.timer.getTime())%6)+1][1])
			else
				love.graphics.setColor(vis_labbgcolors[tilc+1][1])
			end
			love.graphics.rectangle("line", screenoffset+x-.5, y-.5, w, h)
			love.graphics.rectangle("line", screenoffset+x+.5, y+.5, w-2, h-2)
		end
	elseif bg == "warphv" then
		for bgy = -1, 14 do
			for bgx = -1, 19 do
				love.graphics.draw(tilesets[tilesetnames[2]]["img"], tilesets[tilesetnames[2]]["tiles"][720+3*(tilesetblocks[tils].colors[tilc].warpbg - 1)+(80*(metadata.warpdir - 1))], screenoffset+(32*bgx) + (metadata.warpdir == 1 and (32-warpbganimation) or 0), (32*bgy) + (metadata.warpdir == 2 and (32-warpbganimation) or 0), 0, 2)
				love.graphics.draw(tilesets[tilesetnames[2]]["img"], tilesets[tilesetnames[2]]["tiles"][721+3*(tilesetblocks[tils].colors[tilc].warpbg - 1)+(80*(metadata.warpdir - 1))], screenoffset+(32*bgx)+16 + (metadata.warpdir == 1 and (32-warpbganimation) or 0), (32*bgy) + (metadata.warpdir == 2 and (32-warpbganimation) or 0), 0, 2)
				love.graphics.draw(tilesets[tilesetnames[2]]["img"], tilesets[tilesetnames[2]]["tiles"][760+3*(tilesetblocks[tils].colors[tilc].warpbg - 1)+(80*(metadata.warpdir - 1))], screenoffset+(32*bgx) + (metadata.warpdir == 1 and (32-warpbganimation) or 0), (32*bgy)+16 + (metadata.warpdir == 2 and (32-warpbganimation) or 0), 0, 2)
				love.graphics.draw(tilesets[tilesetnames[2]]["img"], tilesets[tilesetnames[2]]["tiles"][761+3*(tilesetblocks[tils].colors[tilc].warpbg - 1)+(80*(metadata.warpdir - 1))], screenoffset+(32*bgx)+16 + (metadata.warpdir == 1 and (32-warpbganimation) or 0), (32*bgy)+16 + (metadata.warpdir == 2 and (32-warpbganimation) or 0), 0, 2)
			end
		end
	elseif bg == "warpa" then
		for squarel = 11, 0, -1 do
			-- Centerx: 128+320 = 448
			local side = (squarel-1)*64 + (warpbganimation*2)

			if squarel % 2 == 0 then
				love.graphics.setColor(warpbgcolors[tilesetblocks[tils].colors[tilc].warpbg][2])
			else
				love.graphics.setColor(warpbgcolors[tilesetblocks[tils].colors[tilc].warpbg][1])
			end

			if side >= 0 then
				love.graphics.rectangle("fill", screenoffset+(320-(side/2)), 240-(side/2), side, side)
			end
		end
	end

	love.graphics.setColor(255,255,255)
end

function vis_is_uncollected(list, x, y)
	return not list[1200*((roomy*20)+roomx) + (y*40)+x]
end

function vis_mark_collected(list, x, y)
	list[1200*((roomy*20)+roomx) + (y*40)+x] = true
end

function vis_will_not_warp(dir)
	local roomdir = levelmetadata[(roomy)*20 + (roomx+1)].warpdir

	if not vis_warp_lines_exist and (roomdir == dir or roomdir == 3) then
		return false
	end

	if vis_player.roomwarp_imminent == dir or vis_player.roomwarp_imminent == 3 then
		return false
	end

	return true
end

function vis_find_crewmate(color, rescuable, either)
	if either == nil then
		-- Either rescuable or internal crewmates
		either = false
	end
	if color == "cyan" or color == "customcyan" or color == "player" then
		return vis_player
	end
	for k,v in pairs(vis_unsolid_entities) do
		if getmetatable(v) == vPlayer and v.color == color
		and (either or v.rescuable == rescuable) then
			return v
		end
	end
	--return vis_player
end

function vis_playmusic(song, simplified)
	-- Take special care with this one, don't let broken music ruin the experience!
	pcall(function()
		-- If this is a simplified number, change it to an internal one.
		cons("vis_playmusic being called to change song to " .. anythingbutnil(song) .. ", " .. (simplified and "simplified" or "internal"))
		if tonumber(song) == nil then
			song = song:gsub("[^%d]", "")
		end
		song = tonumber(song)
		if simplified then
			song = vis_getinternalsongnumber(song)
		end

		cons("Resulting internal song number is " .. song)

		-- The song must not already be playing!
		if song == vis_currentmusic then
			return
		end

		-- Okay, stop the currently playing music
		vis_stopmusic()

		-- The song must exist!
		if vis_musicnumbers[song] == nil then
			return
		end

		-- Playing it is easy.
		vis_musicnumbers[song]:play()
		vis_currentmusic = song
	end)
end

function vis_stopmusic()
	pcall(function()
		vis_musicnumbers[vis_currentmusic]:stop()
	end)
	vis_currentmusic = nil
end

function vis_getinternalsongnumber(simplified_number)
	return ({1,2,3,4,6,8,10,11,12,13,14})[simplified_number]
end

function vis_return_to_editor()
	if vis_return_to_6 then
		tostate(6)
		vis_return_to_6 = false
		state6old1 = false
	else
		tostate(1, true)
	end
	love.keyboard.setKeyRepeat(true)
	vis_stopmusic()
end
