vScripting =
{
	script = nil,
	line = 1,
	active = false,
	keywait = false,
	nocontrol = false,

	text_color = "gray",
	text_x = 0,
	text_y = 0,
	text_content = {},
	text_backgroundtext = false,

	do_script = nil,
	do_line = nil,
	do_count = 0,

	counter = 0,

	currentscript = {},
	transl_script = nil,
	transl_line = 1,
	transl_addedbars = false,
	squeak_enabled = true,

	flags = {},
}

function vScripting:new(o)
	o = o or {}
	setmetatable(o, self)
	self.__index = self
	return o
end

function vScripting:loadscript(script, line, loadedfromscript)
	if line == nil then line = 1 end

	self.script = script
	self.line = line

	self:translate(script, loadedfromscript)

	--[[
	cons("I translated " .. script .. "!\n===")
	for k,v in pairs(self.currentscript) do
		print(v)
	end
	cons("===")
	]]

	self.active = true
end

function vScripting:load_manually(scriptarray)
	self.script = nil
	self.line = 1
	self.currentscript = scriptarray

	self.active = true
end

function vScripting:kill()
	self.active = false
end

function vScripting:blocking()
	if self.active or self.nocontrol then
		return true
	end
	return false
end

function vScripting:keypress()
	if self.keywait then
		self.keywait = false
	end
end

function vScripting:update(dt)
	if not self.active then
		return
	end
	if self.keywait then
		-- We have control over the player now, so we have to use the brakes!
		vis_player:still(dt)
		return
	end

	local yield, moving_player = false
	while not yield do
		-- Now start parsing the current scripting line. But is there any?
		if self.currentscript == nil or self.currentscript[self.line] == nil then
			self.active = false
			return
		end
		local scriptline = string.gsub(string.gsub(string.gsub(self.currentscript[self.line], "%(", ","), "%)", ","), " ", ""):lower()
		local parts = explode(",", scriptline)
		local command = parts[1]
		table.remove(parts, 1)

		local noadvance
		if vis_commands[command] ~= nil then
			yield, noadvance, moving_player = vis_commands[command](dt, unpack(parts))
		end

		-- If we didn't want to advance to the next line, we would have said so!
		if not noadvance then
			self.line = self.line + 1
		end
	end

	-- Again, if we're not moving the player, slow it down!
	if not moving_player then
		vis_player:still(dt)
	end
end

function vScripting:translate(script, loadedfromscript)
	self.currentscript = {}
	self.transl_script = script
	self.transl_line = 1
	self.transl_addedbars = false
	self.squeak_enabled = true

	if scripts[self.transl_script] == nil or #scripts[self.transl_script] == 0 then
		return
	end

	local intext, finished = false, false
	local sticking_last_line = ""
	while not finished do
		-- Translate the current line from simplified into internal scripting - if there is any
		if scripts[self.transl_script] == nil or scripts[self.transl_script][self.transl_line] == nil then
			finished = true
			break
		end
		local scriptline = string.gsub(string.gsub(string.gsub(scripts[self.transl_script][self.transl_line], "%(", ","), "%)", ","), " ", ""):lower()
		local parts = explode(",", scriptline)
		local command = parts[1]
		table.remove(parts, 1)

		if vis_simple_commands[command] ~= nil then
			if intext and command ~= "say" and command ~= "reply" then
				table.insert(self.currentscript, "endtext")
				intext = false
			end

			local add
			add, intext = vis_simple_commands[command](unpack(parts))

			if add ~= nil then
				table.insert(self.currentscript, add)
			end

			if intext and not self.transl_addedbars then
				table.insert(self.currentscript, 1, "cutscene()")
				table.insert(self.currentscript, 2, "untilbars()")
				self.transl_addedbars = true
			end
		end
		sticking_last_line = scriptline

		self.transl_line = self.transl_line + 1
	end

	if intext then
		table.insert(self.currentscript, "endtext")
	end

	if self.transl_addedbars then
		table.insert(self.currentscript, "endcutscene()")
		table.insert(self.currentscript, "untilbars()")
	end

	if loadedfromscript then
		-- First run the last line as internal scripting!
		--table.insert(self.currentscript, 1, scripts[self.transl_script][#scripts[self.transl_script]])
		table.insert(self.currentscript, 1, sticking_last_line)
	end
end

-- These commands either return a single line to add, or, if they have a better idea, return nil.
vis_simple_commands =
{
	say = function(l, c)
		if l == nil then l = 1 end
		if c == nil or c == "" then
			c = "gray"
		elseif c == "1" or c == "viridian" or c == "player" then
			c = "cyan"
		elseif c == "2" or c == "violet" or c == "pink" then
			c = "purple"
		elseif c == "3" or c == "vitellary" then
			c = "yellow"
		elseif c == "4" or c == "vermilion" then -- "vermillion" does not work in VVVVVV
			c = "red"
		elseif c == "5" or c == "verdigris" then
			c = "green"
		elseif c == "6" or c == "victoria" then
			c = "blue"
		end
		local iscrew = c == "cyan" or c == "purple" or c == "yellow" or c == "red" or c == "green" or c == "blue"
		if vis_scripting.squeak_enabled then
			if iscrew then
				table.insert(vis_scripting.currentscript, "squeak(" .. c .. ")")
			else
				table.insert(vis_scripting.currentscript, "squeak(terminal)")
			end
		end
		table.insert(vis_scripting.currentscript, "text(" .. c .. ",0,0," .. l .. ")")
		if tonumber(l) < 0 then
			-- Secret activated
			table.insert(vis_scripting.currentscript, scripts[vis_scripting.transl_script][vis_scripting.transl_line+1])
			vis_scripting.transl_line = vis_scripting.transl_line + 1
		else
			for i = 1, tonumber(l) do
				table.insert(vis_scripting.currentscript, scripts[vis_scripting.transl_script][vis_scripting.transl_line+i])
			end
			vis_scripting.transl_line = vis_scripting.transl_line + tonumber(l)
		end
		if iscrew then
			table.insert(vis_scripting.currentscript, "customposition(" .. c  .. ",above)")
		else
			table.insert(vis_scripting.currentscript, "customposition(center)")
		end
		table.insert(vis_scripting.currentscript, "speak_active")
		return nil, true
	end,
	reply = function(l)
		if l == nil then l = 1 end
		if vis_scripting.squeak_enabled then
			table.insert(vis_scripting.currentscript, "squeak(player)")
		end
		table.insert(vis_scripting.currentscript, "text(cyan,0,0," .. l .. ")")
		if tonumber(l) < 0 then
			-- Secret activated
			table.insert(vis_scripting.currentscript, scripts[vis_scripting.transl_script][vis_scripting.transl_line+1])
			vis_scripting.transl_line = vis_scripting.transl_line + 1
		else
			for i = 1, tonumber(l) do
				table.insert(vis_scripting.currentscript, scripts[vis_scripting.transl_script][vis_scripting.transl_line+i])
			end
			vis_scripting.transl_line = vis_scripting.transl_line + tonumber(l)
		end
		table.insert(vis_scripting.currentscript, "position(player,above)")
		table.insert(vis_scripting.currentscript, "speak_active")
		return nil, true

	end,
	delay = function(ticks)
		if ticks == nil then return end
		return "delay(" .. ticks .. ")"
	end,
	destroy = function(ob)
		if ob == nil then return end
		return "destroy(" .. ob .. ")"
	end,
	music = function(song)
		-- NOTE: Should music(9a) or similar ever result in music(9) and thus the wrong internal number,
		-- check whether tonumber("9a") is actually nil now!
		if song == nil then return end
		num_song = tonumber(song)
		if num_song == nil then
			return "play(" .. song .. ")"
		elseif num_song == 0 then
			return "stopmusic"
		elseif num_song >= 1 and num_song <= 11 then
			return "play(" .. vis_getinternalsongnumber(num_song) .. ")"
		else
			return "play(" .. song:gsub("[^%d]", "") .. ")"
		end
	end,
	playremix = function()
		return "play(15)"
	end,
	flash = function()
		table.insert(vis_scripting.currentscript, "flash(5)")
		table.insert(vis_scripting.currentscript, "shake(20)")
		table.insert(vis_scripting.currentscript, "playef(9,10)")
	end,
	happy = function(c)
		-- :D
		-- changecustommood(customcyan,0)
		-- squeak(player)
		-- This is even more different in VVVVVV :)
		if c == nil then
			c = "player"
		elseif c == "1" or c == "viridian" or c == "player" then
			c = "cyan"
		elseif c == "2" or c == "violet" or c == "pink" then
			c = "purple"
		elseif c == "3" or c == "vitellary" then
			c = "yellow"
		elseif c == "4" or c == "vermilion" then -- "vermillion" does not work in VVVVVV
			c = "red"
		elseif c == "5" or c == "verdigris" then
			c = "green"
		elseif c == "6" or c == "victoria" then
			c = "blue"
		end
		-- TODO: all/everyone/everybody: how does that even get translated?
		table.insert(vis_scripting.currentscript, "changecustommood(" .. c .. ",0)")
		table.insert(vis_scripting.currentscript, "squeak(" .. c .. ")")
	end,
	sad = function(c)
		-- D:
		-- This is even more different in VVVVVV :(
		if c == nil then
			c = "player"
		elseif c == "1" or c == "viridian" or c == "player" then
			c = "cyan"
		elseif c == "2" or c == "violet" or c == "pink" then
			c = "purple"
		elseif c == "3" or c == "vitellary" then
			c = "yellow"
		elseif c == "4" or c == "vermilion" then -- "vermillion" does not work in VVVVVV
			c = "red"
		elseif c == "5" or c == "verdigris" then
			c = "green"
		elseif c == "6" or c == "victoria" then
			c = "blue"
		end
		-- TODO: all/everyone/everybody: how does that even get translated?
		table.insert(vis_scripting.currentscript, "changecustommood(" .. c .. ",1)")
		table.insert(vis_scripting.currentscript, "squeak(cry)")
	end,
	flag = function(fl, val)
		if fl == nil or val == nil then return end
		return "flag(" .. fl .. "," .. val .. ")"
	end,
	ifflag = function(fl, script)
		if fl == nil or script == nil then return end
		return "customifflag(" .. fl .. "," .. script .. ")"
	end,
	iftrinkets = function(amount, script)
		if amount == nil or script == nil then return end
		return "customiftrinkets(" .. amount .. "," .. script .. ")"
	end,
	iftrinketsless = function(amount, script)
		if amount == nil or script == nil then return end
		return "customiftrinketsless(" .. amount .. "," .. script .. ")"
	end,
	map = function(arg)
		return "custommap(" .. arg .. ")"
	end,
	squeak = function(squeaker)
		if squeaker == "on" then
			vis_scripting.squeak_enabled = true
		elseif squeaker == "off" then
			vis_scripting.squeak_enabled = false
		else
			return "squeak(" .. squeaker .. ")"
		end
	end,
	-- TODO: speaker(x)
}

vis_commands =
{
	moveplayer = function(_, x, y)
		vis_player:teleportrelative(tonumber(x), tonumber(y))
	end,
	delay = function(dt, ticks)
		vis_scripting.counter = vis_scripting.counter + 30*dt
		if vis_scripting.counter > tonumber(ticks) then
			vis_scripting.counter = 0
			return
		end
		return true, true -- yield, don't advance to the next command
	end,
	flash = function(_, ticks)
		vis_flashtimer = tonumber(ticks)
	end,
	shake = function(_, ticks)
		vis_shaketimer = tonumber(ticks)
	end,
	walk = function(dt, dir, ticks)
		vis_scripting.counter = vis_scripting.counter + 30*dt
		if vis_scripting.counter > tonumber(ticks) then
			vis_scripting.counter = 0
			return
		end
		if dir == "left" then
			vis_player:left(dt)
		elseif dir == "right" then
			vis_player:right(dt)
		end
		return true, true, true -- yield, don't advance, and we're moving the player
	end,
	flip = function()
		vis_player:jump()
	end,
	tofloor = function()
		vis_player:tofloor()
	end,
	playef = function(_, ef)
		if vis_effectnumbers[tonumber(ef)] ~= nil then
			vis_effectnumbers[tonumber(ef)]:stop()
			vis_effectnumbers[tonumber(ef)]:play()
		end
	end,
	play = function(_, song)
		vis_playmusic(song)
	end,
	stopmusic = function()
		vis_stopmusic()
	end,
	resumemusic = function()
		-- !!!
	end,
	musicfadeout = function()
		-- TODO: fadeout
		vis_stopmusic()
	end,
	musicfadein = function()
		-- !!!
	end,
	gotoposition = function(_, x, y, z)
		vis_player:teleport(tonumber(x)+6, tonumber(y))
		vis_player:setflipped(z == "1")
	end,
	gotoroom = function(_, x, y)
		gotoroom(tonumber(x), tonumber(y))
		vis_init_room()
	end,
	cutscene = function()
		vis_cutscenebars_dir = true
	end,
	endcutscene = function()
		vis_cutscenebars_dir = false
	end,
	untilbars = function()
		if (not vis_cutscenebars_dir and vis_cutscenebars == 0)
		or (vis_cutscenebars_dir and vis_cutscenebars == 320) then
			return
		end
		return true, true -- yield and don't advance to the next line
	end,
	text = function(_, color, x, y, l)
		vis_scripting.text_color = color
		vis_scripting.text_x = tonumber(x)
		vis_scripting.text_y = tonumber(y)
		vis_scripting.text_content = {}
		vis_scripting.text_backgroundtext = false

		for i = 1, tonumber(l) do
			vis_scripting.line = vis_scripting.line + 1
			assert(vis_scripting.currentscript[vis_scripting.line] ~= nil)
			table.insert(vis_scripting.text_content, vis_scripting.currentscript[vis_scripting.line])
		end

		-- There are a few magic values of x and y
		if x == "-1" or x == "-500" then
			vis_commands.position(nil, "centerx")
		end
		if y == "-500" then
			vis_commands.position(nil, "centery")
		end
	end,
	position = function(_, a, b)
		local maxwidth = 0
		for k,v in pairs(vis_scripting.text_content) do
			if v:len() > maxwidth then
				maxwidth = v:len()
			end
		end
		if a == "centerx" or a == "center" then
			-- Center horizontally
			vis_scripting.text_x = (304-maxwidth*8)/2
		end
		if a == "centery" or a == "center" then
			-- Center vertically
			vis_scripting.text_y = (224-(#vis_scripting.text_content*8))/2
		end

		local villi = vis_find_crewmate(a, false)

		if villi ~= nil then
			if villi.dir == 0 then
				vis_scripting.text_x = villi.x - 16
			else
				vis_scripting.text_x = villi.x - 8 - maxwidth*8
			end
			if b == "above" then
				vis_scripting.text_y = villi.y - 18 - (#vis_scripting.text_content*8)
			else
				vis_scripting.text_y = villi.y + 26
			end
		end
	end,
	customposition = function(_, a, b)
		local maxwidth = 0
		for k,v in pairs(vis_scripting.text_content) do
			if v:len() > maxwidth then
				maxwidth = v:len()
			end
		end
		if a == "centerx" or a == "center" then
			-- Center horizontally
			vis_scripting.text_x = (304-maxwidth*8)/2
		end
		if a == "centery" or a == "center" then
			-- Center vertically
			vis_scripting.text_y = (224-(#vis_scripting.text_content*8))/2
		end

		local villi = vis_find_crewmate(a, true)

		if villi ~= nil then
			if villi.dir == 0 then
				vis_scripting.text_x = villi.x - 16
			else
				vis_scripting.text_x = villi.x - 8 - maxwidth*8
			end
			if b == "above" then
				vis_scripting.text_y = villi.y - 18 - (#vis_scripting.text_content*8)
			else
				vis_scripting.text_y = villi.y - 26
			end
		end
	end,
	backgroundtext = function()
		vis_scripting.text_backgroundtext = true
	end,
	flipme = function()
		-- Flip mode
		if false then
			vis_scripting.text_y = vis_scripting.text_y + 2*(120-vis_scripting.text_y);
		end
	end,
	speak_active = function()
		vvvvvv_textboxes = {}
		table.insert(vvvvvv_textboxes, {vis_scripting.text_color, vis_scripting.text_x, vis_scripting.text_y, vis_scripting.text_content})
		if not vis_scripting.text_backgroundtext then
			vis_scripting.keywait = true
			return true -- yield
		end
	end,
	speak = function()
		table.insert(vvvvvv_textboxes, {vis_scripting.text_color, vis_scripting.text_x, vis_scripting.text_y, vis_scripting.text_content})
		if not vis_scripting.text_backgroundtext then
			vis_scripting.keywait = true
			return true -- yield
		end
	end,
	endtext = function()
		vvvvvv_textboxes = {}
		-- End non-fast?
	end,
	endtextfast = function()
		vvvvvv_textboxes = {}
	end,
	["do"] = function(_, count)
		vis_scripting.do_script = vis_scripting.script
		vis_scripting.do_line = vis_scripting.line -- The loop statement does advance the line number, so adding 1 is not necessary here.
		vis_scripting.do_count = tonumber(count)
	end,
	loop = function()
		vis_scripting.do_count = vis_scripting.do_count - 1
		if vis_scripting.do_count ~= 0 then -- Not <= 0, no...
			vis_scripting:loadscript(vis_scripting.do_script, vis_scripting.do_line, false)
		end
	end,
	vvvvvvman = function()
		-- !!!
	end,
	undovvvvvvman = function()
		-- !!!
	end,
	createentity = function()
		-- !!! TODO, at least partially
	end,
	createcrewman = function(_, x, y, color, mood, ai, ai2)
		table.insert(vis_unsolid_entities, vPlayer:new{
				x=tonumber(x)+6, y=tonumber(y),
				color=color,
				defaultcolor=defaultcolor,
				mood=tonumber(mood),
				ai=ai, ai2=tonumber(ai2),
				is_main_player=false,
				rescuable=false,
				flippable=true
			}
		)
		-- Also mess up the current text box color!
		local origcolor = textboxcolors[vis_scripting.text_color]
		if origcolor == nil then
			origcolor = textboxcolors.gray
		end
		vis_scripting.text_color = {0, origcolor[2], origcolor[3]}
	end,
	changemood = function(_, color, mood)
		local villi = vis_find_crewmate(color, false)

		if villi ~= nil then
			villi.mood = tonumber(mood)
		end
	end,
	changecustommood = function(_, color, mood)
		local villi = vis_find_crewmate(color, true)

		if villi ~= nil then
			villi.mood = tonumber(mood)
		end
	end,
	changetile = function(_, color, tile)
		local villi = vis_find_crewmate(color, false)

		if villi ~= nil then
			-- VVVVVV's tile system is somewhat... odd.
			local supposed_tile = 3*villi.dir
			local subtract_this = 144*villi.mood + (villi.flipped and 6 or 0)
			villi.tileoffset = tonumber(tile - subtract_this)
		end
	end,
	flipgravity = function(_, color)
		local villi = vis_find_crewmate(color, false)

		if villi ~= nil then
			villi:jump(true)
			villi.mood = 0 -- tiles and such
		end
	end,
	changegravity = function(_, color)
		local villi = vis_find_crewmate(color, false)

		if villi ~= nil then
			villi.tile = villi.tile + 12
		end
	end,
	changedir = function(_, color, dir)
		local villi = vis_find_crewmate(color, false)

		if villi ~= nil then
			villi.ai = nil
			villi.dir = tonumber(dir)
		end
	end,
	alarmon = function()
		viss_crashing_repeat:play()
	end,
	alarmoff = function()
		viss_crashing_repeat:stop()
	end,
	changeai = function(_, color, ai1, ai2)
		local villi = vis_find_crewmate(color, false)

		if villi ~= nil then
			villi.ai = ai1
			villi.ai2 = tonumber(ai2)
		end
	end,
	activateteleporter = function()
		-- !!!
	end,
	changecolour = function(_, c1, c2)
		local villi = vis_find_crewmate(c1, false)

		if villi ~= nil then
			villi.color = c2
		end
	end,
	squeak = function(_, s)
		local sound
		if s == "cry" then
			sound = viss_hurt
		elseif s == "terminal" then
			sound = viss_blip2
		elseif s == "player" or s == "cyan" then
			sound = viss_crew1
		elseif s == "green" then
			sound = viss_crew2
		elseif s == "blue" then
			sound = viss_crew3
		elseif s == "yellow" then
			sound = viss_crew4
		elseif s == "purple" then
			sound = viss_crew5
		elseif s == "red" then
			sound = viss_crew6
		end
		if sound ~= nil then
			sound:stop()
			sound:play()
		end
	end,
	blackout = function()
		-- !!!
	end,
	blackon = function()
		-- !!!
	end,
	setcheckpoint = function()
		vis_player:setcheckpoint()
	end,
	gamestate = function()
		-- !!!
	end,
	textboxactive = function()
		vvvvvv_textboxes = {vvvvvv_textboxes[#vvvvvv_textboxes]}
	end,
	gamemode = function()
		-- !!!
	end,
	ifexplored = function()
		-- !!!
	end,
	iflast = function()
		-- !!!
	end,
	ifskip = function()
		-- !!!
	end,
	ifflag = function(_, fl)
		if vis_scripting.flags[tonumber(fl)] then
			vis_scripting:kill()
			return true -- yield
		end
	end,
	ifcrewlost = function()
		-- !!!
	end,
	iftrinkets = function(_, amount)
		if vis_n_trinkets >= tonumber(amount) then
			vis_scripting:kill()
			return true -- yield
		end
	end,
	iftrinketsless = function(_, amount)
		if vis_n_trinkets < tonumber(amount) then
			vis_scripting:kill()
			return true -- yield
		end
	end,
	hidecoordinates = function()
		-- !!!
	end,
	showcoordinates = function()
		-- !!!
	end,
	hideship = function()
		-- !!!
	end,
	showship = function()
		-- !!!
	end,
	showsecretlab = function()
		-- !!!
	end,
	hidesecretlab = function()
		-- !!!
	end,
	showteleporters = function()
		-- !!!
	end,
	showtargets = function()
		-- !!!
	end,
	showtrinkets = function()
		-- !!!
	end,
	hideteleporters = function()
		-- !!!
	end,
	hidetargets = function()
		-- !!!
	end,
	hidetrinkets = function()
		-- !!!
	end,
	hideplayer = function()
		vis_player:hide()
	end,
	showplayer = function()
		vis_player:show()
	end,
	teleportscript = function()
		-- !!!
	end,
	clearteleportscript = function()
		-- !!!
	end,
	nocontrol = function()
		vis_scripting.nocontrol = true
	end,
	hascontrol = function()
		vis_scripting.nocontrol = false
	end,
	companion = function()
		-- !!!
	end,
	befadein = function()
		-- !!!
	end,
	fadein = function()
		-- !!!
	end,
	fadeout = function()
		-- !!!
	end,
	untilfade = function()
		-- !!!
	end,
	entersecretlab = function()
		-- !!!
	end,
	leavesecretlab = function()
		-- !!!
	end,
	resetgame = function()
		-- !!!
	end,
	loadscript = function()
		vis_scripting:kill()
		return true -- yield
	end,
	rollcredits = function()
		-- !!!
	end,
	finalmode = function()
		-- !!!
	end,
	rescued = function()
		-- !!!
	end,
	missing = function()
		-- !!!
	end,
	face = function(_, c1, c2)
		local villi = vis_find_crewmate(c1, false)

		if villi ~= nil then
			villi.ai = "face" .. c2
		end
	end,
	jukebox = function()
		-- !!!
	end,
	createactivityzone = function()
		-- !!!
	end,
	createrescuedcrew = function()
		-- !!!
	end,
	restoreplayercolour = function()
		vis_player.color = "cyan"
	end,
	changeplayercolour = function(_, c)
		vis_player.color = c
	end,
	altstates = function()
		-- !!!
	end,
	activeteleporter = function()
		-- !!!
	end,
	foundtrinket = function()
		-- !!!
	end,
	foundlab = function()
		-- !!!
	end,
	foundlab2 = function()
		-- !!!
	end,
	everybodysad = function()
		vis_player.mood = 1
		for k,v in pairs(vis_unsolid_entities) do
			if getmetatable(v) == vPlayer then
				v.mood = 1
			end
		end
	end,
	startintermission2 = function()
		-- !!!
	end,
	telesave = function()
		-- !!!
	end,
	createrescued = function()
		-- !!!
	end,
	specialline = function(_, case)
		if case == "1" then
			vis_scripting.text_content[1] = "I'm worried about you, Doctor!"
		elseif case == "2" then
			if #vis_scripting.text_content == 0 then
				vis_scripting.text_content[1] = "cutscene()"
			end
			vis_scripting.text_content[2] = "to helping you find you!"
		end
	end,
	trinketbluecontrol = function()
		-- !!!
	end,
	trinketyellowcontrol = function()
		-- !!!
	end,
	redcontrol = function()
		-- !!!
	end,
	greencontrol = function()
		-- !!!
	end,
	bluecontrol = function()
		-- !!!
	end,
	yellowcontrol = function()
		-- !!!
	end,
	purplecontrol = function()
		-- !!!
	end,
	customifflag = function(_, fl, script)
		if vis_scripting.flags[tonumber(fl)] then
			vis_scripting:loadscript(script, nil, true)
			return true, true
		end
	end,
	customiftrinkets = function(_, amount, script)
		if vis_n_trinkets >= tonumber(amount) then
			vis_scripting:loadscript(script, nil, true)
			return false, true -- Don't go to line 2 in the new script
		end
	end,
	customiftrinketsless = function(_, amount, script)
		-- I'll ignore that the command is broken in VVVVVV
		if vis_n_trinkets < tonumber(amount) then
			vis_scripting:loadscript(script, nil, true)
			return false, true
		end
	end,
	destroy = function(_, ob)
		local entitytable, expectedclass
		if ob == "gravitylines" then
			entitytable, expectedclass = vis_unsolid_entities, vGravityline
		elseif ob == "warptokens" then
			entitytable, expectedclass = vis_unsolid_entities, vWarptoken
		elseif ob == "platforms" then
			entitytable, expectedclass = vis_solid_entities, vMovingPlat
		else
			return
		end
		for k,v in pairs(entitytable) do
			if getmetatable(v) == expectedclass then
				v:destroy()
			end
		end
	end,
	flag = function(_, fl, val)
		if tonumber(fl) >= 0 and tonumber(fl) <= 99 then
			vis_scripting.flags[tonumber(fl)] = (val == "on")
		end
	end,

	vis_playerfreeze = function(val)
		vis_player.frozen = val == "1"
	end,
	vis_checkfinished = function()
		if vis_crewmates_total - vis_n_crewmates == 0 then
			vis_return_to_editor()
		end
	end,
}
