vMovingPlat =
{
	x = 0,
	y = 0,
	w = 32,
	h = 8,
	p1 = 0,
	destroyed = false,
	currentdir = 0,

	attached_ents = {},
}

function vMovingPlat:new(o)
	o = o or {}
	setmetatable(o, self)
	self.__index = self

	o.currentdir = o.p1

	return o
end

function vMovingPlat:draw(xoff, yoff)
	if self.destroyed then
		return
	end

	love.graphics.setColor(
		tilesetblocks[levelmetadata[(roomy)*20 + (roomx+1)].tileset].colors[levelmetadata[(roomy)*20 + (roomx+1)].tilecol].entcolor
	)
	love.graphics.draw(platformimg, platformpart[1], xoff+screenoffset+self.x*2, yoff+self.y*2, 0, 2)
	love.graphics.draw(platformimg, platformpart[2], xoff+screenoffset+self.x*2 + 16, yoff+self.y*2, 0, 2)
	love.graphics.draw(platformimg, platformpart[2], xoff+screenoffset+self.x*2 + 32, yoff+self.y*2, 0, 2)
	love.graphics.draw(platformimg, platformpart[3], xoff+screenoffset+self.x*2 + 48, yoff+self.y*2, 0, 2)

	love.graphics.setColor(255, 255, 255)
end

function vMovingPlat:update(dt)
	if self.destroyed or vis_player.frozen or vis_player:isdead() then
		return
	end

	-- MOVE: 0 1 2 3 - onder boven links rechts (richting)

	local movement = 30*levelmetadata[(roomy)*20 + (roomx+1)].platv*dt
	local dx, dy = 0, 0

	if self.currentdir == 0 then
		dy = movement
	elseif self.currentdir == 1 then
		dy = -movement
	elseif self.currentdir == 2 then
		dx = -movement
	elseif self.currentdir == 3 then
		dx = movement
	end

	-- First make sure my dx and dy are actually possible.
	dx, dy = self:checkcollision(dx, dy, dt)

	-- Actually move
	self.x = self.x + dx
	self.y = self.y + dy

	for k,v in pairs(self.attached_ents) do
		if v.attached_to == self then
			v:correctcollision(v.x+dx, v.y+dy, dt)
		end
	end
end

function vMovingPlat:checkcollision(dx, dy, dt)
	-- This is different from vPlayer:correctcollision(new_x, new_y, dt), given it only returns new
	-- delta values. This is needed because we also need to move a player if it's standing on this.
	-- Also inverts the current direction.
	if dx == 0 and dy == 0 then
		return 0, 0
	end

	local necessary_steps = math.max(
		math.ceil(math.abs(dx)/8),
		math.ceil(math.abs(dy)/8)
	)
	local is_inside_wall = false
	local hit_dx, hit_dy
	for step = 1, necessary_steps do
		local step_newx = self.x+((dx/necessary_steps)*step)
		local step_newy = self.y+((dy/necessary_steps)*step)
		if self:isinsidewall(step_newx, step_newy, dx, dy) then
			is_inside_wall = true
			-- hit_d* is the point at which the collision first occurred,
			-- we might have tried to go even further! (on low FPS)
			hit_dx, hit_dy = step_newx-self.x, step_newy-self.y
			break
		end
	end

	if is_inside_wall then
		-- So how much have we gone too far? (Won't be more than 8 pixels due to the steps above)
		local compensate_dx, compensate_dy = 0, 0
		if dx < 0 then
			-- Going left
			compensate_dx = ((math.ceil(hit_dx/8)*8 - hit_dx)*2) + (hit_dx-dx)
		elseif dx > 0 then
			-- Going right
			compensate_dx = -((hit_dx - math.floor(hit_dx/8)*8)*2) + (dx-hit_dx)
		elseif dy < 0 then
			-- Going up
			compensate_dy = ((math.ceil(hit_dy/8)*8 - hit_dy)*2) + (hit_dy-dy)
		elseif dy > 0 then
			-- Going down
			compensate_dy = -((hit_dy - math.floor(hit_dy/8)*8)*2) + (dy-hit_dy)
		end

		dx, dy = dx+compensate_dx, dy+compensate_dy

		-- Invert the current direction as well.
		if self.currentdir == 0 or self.currentdir == 2 then
			self.currentdir = self.currentdir + 1
		else
			self.currentdir = self.currentdir - 1
		end
	end

	return dx, dy
end

function vMovingPlat:isinsidewall(custom_x, custom_y, dx, dy)
	-- Just returns true or false
	if custom_x == nil then custom_x = self.x end
	if custom_y == nil then custom_y = self.y end
	local vertical_movement = dy ~= 0

	local md = levelmetadata[(roomy)*20 + (roomx+1)]

	-- Are we now going outside platform bounds? Or the entire level?
	if vertical_movement and (custom_y < md.platy1 or custom_y+self.h > md.platy2) then
		return true
	end
	if not vertical_movement and (custom_x < md.platx1 or custom_x+self.w > md.platx2) then
		return true
	end

	-- Then maybe look at tiles
	local tilecoords
	if dx < 0 then
		-- Going left
		tilecoords = { {math.floor(custom_x/8), custom_y/8} }
	elseif dx > 0 then
		-- Going right
		tilecoords = { {math.floor((custom_x+self.w)/8), custom_y/8} }
	elseif dy < 0 then
		-- Going up
		local ty = math.floor(custom_y/8)
		tilecoords = {
			{custom_x/8, ty},
			{custom_x/8+1, ty},
			{custom_x/8+2, ty},
			{custom_x/8+3, ty}
		}
	elseif dy > 0 then
		-- Going down
		local ty = math.floor((custom_y+self.h)/8)
		tilecoords = {
			{custom_x/8, ty},
			{custom_x/8+1, ty},
			{custom_x/8+2, ty},
			{custom_x/8+3, ty}
		}
	end

	for _, t in pairs(tilecoords) do
		local tilenum = math.floor(t[2]*40+t[1]+1)
		if issolid(roomdata[roomy][roomx][tilenum], usedtilesets[md.tileset], vis_player.invincible, false) then
			return true
		end
	end

	return false
end

function vMovingPlat:destroy()
	self.destroyed = true
end

-- Todo: inheritance
function vMovingPlat:touching(x, y, w, h)
	if self.x >= x+w or self.x+self.w <= x then
		return false
	end
	if self.y >= y+h or self.y+self.h <= y then
		return false
	end
	return true
end

function vMovingPlat:hit(side, hitter, dt)
	-- Health warning: moving platforms in particular were rushed.

	-- SIDE: 1 2 3 4 - onder boven rechts links (van het platform)
	-- MOVE: 0 1 2 3 - onder boven links rechts (richting)
	if side == nil then
		return false
	end
	local renew_attachment = false
	if self.currentdir >= 2 then
		-- Moving sideways
		if side == 1 or side == 2 then 
			-- Move sideways, you know, the mechanic that didn't exist in the VVVVVVVV prototype
			renew_attachment = true
		end
	elseif self.currentdir >= 0 then
		-- Moving up/down
		if side == 1 or side == 2 then
			-- We're standing on it.
			renew_attachment = true
		end
	end

	if renew_attachment then
		local is_attached = false
		for k,v in pairs(self.attached_ents) do
			if v == hitter then
				is_attached = true
			end
		end
		if not is_attached then
			table.insert(self.attached_ents, hitter)
		end
		return true
	end
	return false
end
